"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processor = void 0;
const metadataProcessor_1 = require("./metadataProcessor");
const blocks_1 = require("./blocks");
const tocHelper_1 = require("./tocHelper");
const tableOfContent_1 = require("./tableOfContent");
const processor = async (blocks, actions, getCache, createNodeId, reporter, cache) => {
    const { thumbnail, tableOfContents, updatedBlocks, rawText } = await processBlocksForContent(blocks, actions, getCache, createNodeId, reporter, cache);
    await (0, metadataProcessor_1.processMetadata)(blocks, actions, createNodeId, reporter, cache);
    return [thumbnail, tableOfContents, updatedBlocks, rawText];
};
exports.processor = processor;
const processBlocksForContent = async (blocks, actions, getCache, createNodeId, reporter, cache) => {
    const context = {
        actions,
        getCache,
        createNodeId,
        reporter,
        cache,
    };
    // 목차 컨텍스트 초기화
    (0, tableOfContent_1.resetTocContext)();
    // 블록 프로세서 레지스트리 생성
    const processorRegistry = new blocks_1.BlockProcessorRegistry(context);
    const tableOfContents = [];
    let thumbnail = null;
    let rawText = "";
    const updatedBlocks = [];
    // 첫 번째 이미지 블록을 찾아 썸네일로 사용
    let firstImageIndex = blocks.findIndex((block) => block.type === "image");
    // 헤딩 타입 블록을 먼저 처리하여 구조를 생성
    const headingBlocks = blocks.filter((block) => ["heading_1", "heading_2", "heading_3"].includes(block.type));
    if (headingBlocks.length > 0) {
        reporter.info(`Found ${headingBlocks.length} heading blocks for TOC generation`);
    }
    // 블록 처리
    const processResults = await Promise.all(blocks.map(async (block, index) => {
        const result = await processorRegistry.processBlock(block);
        // 썸네일 처리
        if (index === firstImageIndex && result.thumbnail) {
            thumbnail = result.thumbnail;
        }
        // 텍스트 데이터 추가
        if (result.plainText) {
            rawText += result.plainText + " ";
        }
        // 목차 데이터 추가
        if (result.tableOfContents) {
            tableOfContents.push(...result.tableOfContents);
        }
        return result;
    }));
    // 목차 로깅 및 디버깅
    const h1Count = tableOfContents.filter((item) => item.type === "heading_1").length;
    const h2Count = tableOfContents.filter((item) => item.type === "heading_2").length;
    const h3Count = tableOfContents.filter((item) => item.type === "heading_3").length;
    reporter.info(`TOC entries before optimization: ${tableOfContents.length} total, ${h1Count} h1, ${h2Count} h2, ${h3Count} h3`);
    // 목차 최적화
    const processedToc = (0, tocHelper_1.optimizeTocArray)(tableOfContents, reporter, {
        maxSize: 2000,
        warnThreshold: 500,
        removeDuplicates: true,
        enhanceDuplicates: true,
        buildHierarchy: true,
    });
    // 업데이트된 블록 적용
    processResults.forEach((result, index) => {
        if (result.updatedBlock) {
            updatedBlocks[index] = result.updatedBlock;
        }
        else {
            updatedBlocks[index] = blocks[index];
        }
    });
    return { thumbnail, tableOfContents: processedToc, updatedBlocks, rawText };
};
