"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatTableOfContents = void 0;
/**
 * 목차 구조를 계층적으로 변환하는 함수
 */
const formatTableOfContents = (tableOfContents, reporter) => {
    if (!tableOfContents || tableOfContents.length === 0) {
        return [];
    }
    try {
        // Deep clone to avoid modifying original
        const tocClone = JSON.parse(JSON.stringify(tableOfContents));
        // Ensure TOC entries are sorted by level first, then by appearance order
        const sortedToc = tocClone.sort((a, b) => {
            // Level 기준 정렬
            if ((a.level || 0) !== (b.level || 0)) {
                return (a.level || 0) - (b.level || 0);
            }
            // Original order is preserved
            return 0;
        });
        // Add contextTitle where missing
        for (const entry of sortedToc) {
            if (!entry.contextTitle) {
                entry.contextTitle = entry.title;
            }
        }
        // H1 섹션별 명령어 매핑 & 실제 자식 항목 분석
        const h1Sections = new Map(); // H1 해시 -> H1 항목
        const h1Children = new Map(); // H1 해시 -> H2 자식 해시 세트
        const h2Children = new Map(); // H2 해시 -> H3 자식 해시 세트
        // 1단계: 모든 H1 항목 수집
        for (const entry of sortedToc) {
            if (entry.level === 1) {
                h1Sections.set(entry.hash, entry);
                h1Children.set(entry.hash, new Set());
            }
        }
        // 2단계: 정당한 H2 항목 식별 (H1의 직접 자식)
        for (const entry of sortedToc) {
            if (entry.level === 2 &&
                entry.parentHash &&
                h1Sections.has(entry.parentHash)) {
                // H1의 자식으로 추가
                h1Children.get(entry.parentHash).add(entry.hash);
                h2Children.set(entry.hash, new Set());
            }
        }
        // 3단계: 정당한 H3 항목 식별 (H2의 직접 자식)
        for (const entry of sortedToc) {
            if (entry.level === 3 && entry.parentHash) {
                // 부모가 H2인 경우
                if (h2Children.has(entry.parentHash)) {
                    h2Children.get(entry.parentHash).add(entry.hash);
                }
            }
        }
        // 4단계: 각 H1 섹션 내에서 일반적인 명령어 이름이 실제 내용을 가지는지 확인
        const commandSections = new Map(); // H1 해시 -> (명령어 이름 -> 유효성)
        const duplicateCommandNames = new Set();
        // 모든 섹션에서 명령어 이름 수집
        for (const entry of sortedToc) {
            if (entry.level === 2) {
                // 일반적인 명령어 이름인지 확인
                const isCommonCommand = /^(cp|mv|rm|find|grep|curl|wget|ls|chmod|chown|ps|sed|awk|cat|service|file|locate|stat)$/.test(entry.title.toLowerCase());
                if (isCommonCommand) {
                    duplicateCommandNames.add(entry.title.toLowerCase());
                    // 이 명령어가 실제 H3 자식을 가지는지 확인
                    const hasChildren = !!(entry.parentHash &&
                        h2Children.has(entry.hash) &&
                        h2Children.get(entry.hash).size > 0);
                    // 섹션별 명령어 유효성 추적
                    if (entry.parentHash) {
                        if (!commandSections.has(entry.parentHash)) {
                            commandSections.set(entry.parentHash, new Map());
                        }
                        commandSections
                            .get(entry.parentHash)
                            .set(entry.title.toLowerCase(), hasChildren);
                    }
                }
            }
        }
        // 5단계: 유효하지 않은 명령어 항목 필터링
        const invalidEntries = new Set();
        // Squid 프록시 서버와 같은 특정 섹션의 특별 처리
        const squidSectionEntry = sortedToc.find((entry) => entry.level === 1 && entry.title.includes("Squid 프록시 서버"));
        if (squidSectionEntry) {
            const squidSectionHash = squidSectionEntry.hash;
            const squidCommandStatus = commandSections.get(squidSectionHash);
            // Squid 섹션의 모든 H2 항목 검토
            for (const entry of sortedToc) {
                if (entry.level === 2 && entry.parentHash === squidSectionHash) {
                    // 일반적인 명령어 이름이면서 실제 내용(H3 자식)이 없는 항목은 제거 대상
                    const isCommonCommand = /^(cp|mv|rm|find|grep|curl|wget|ls|chmod|chown|ps|sed|awk|cat|service|file|locate|stat)$/.test(entry.title.toLowerCase());
                    // Squid 섹션 특성에 맞지 않는 일반 명령어 식별
                    if (isCommonCommand) {
                        // 이 명령어가 Squid 섹션에서 정당한 내용을 가지는지 확인
                        const hasValidChildren = !!(entry.hash &&
                            h2Children.has(entry.hash) &&
                            h2Children.get(entry.hash).size > 0);
                        // 내용 없는 명령어는 제거
                        if (!hasValidChildren) {
                            invalidEntries.add(entry.hash);
                            // 이 명령어의 모든 자식도 제거 (혹시 있을 경우)
                            for (const childEntry of sortedToc) {
                                if (childEntry.parentHash === entry.hash) {
                                    invalidEntries.add(childEntry.hash);
                                }
                            }
                        }
                    }
                }
            }
        }
        // 유효하지 않은 항목 제외한 결과 생성
        const validToc = sortedToc.filter((entry) => !invalidEntries.has(entry.hash));
        if (invalidEntries.size > 0) {
            reporter.info(`Removed ${invalidEntries.size} invalid TOC entries from special sections`);
        }
        return validToc;
    }
    catch (error) {
        reporter.error(`Error formatting table of contents: ${error}`);
        return tableOfContents; // Return original on error
    }
};
exports.formatTableOfContents = formatTableOfContents;
