"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.enrichDuplicateTitles = exports.validateTocStructure = exports.optimizeTocArray = void 0;
/**
 * Utility function to efficiently handle large table of contents arrays
 * by removing duplicates and optionally limiting size while preserving hierarchy
 */
const optimizeTocArray = (tocEntries, reporter, options = {}) => {
    const { maxSize = 2000, // Maximum entries to include
    warnThreshold = 300, // Threshold to issue warning
    removeDuplicates = true, // Whether to remove duplicates
    enhanceDuplicates = true, // Whether to enhance duplicate titles with context
    buildHierarchy = true, // Whether to build the hierarchy tree
     } = options;
    if (!tocEntries || tocEntries.length === 0) {
        return [];
    }
    // Track memory usage for large TOCs
    if (tocEntries.length > warnThreshold) {
        reporter.warn(`Large table of contents detected (${tocEntries.length} items). This might affect performance.`);
    }
    // Pre-process all entries to ensure proper levels
    const enrichedTocEntries = tocEntries.map((entry) => {
        if (!entry.level) {
            const level = entry.type === "heading_1"
                ? 1
                : entry.type === "heading_2"
                    ? 2
                    : entry.type === "heading_3"
                        ? 3
                        : 0;
            return { ...entry, level };
        }
        return entry;
    });
    // 구조적 유효성 검증 및 정리
    const validatedToc = (0, exports.validateTocStructure)(enrichedTocEntries, reporter);
    // Remove duplicates if requested
    let processedToc = validatedToc;
    if (removeDuplicates) {
        const startTime = Date.now();
        // 중복 해시 및 타이틀 검사를 위한 맵
        const uniqueMap = new Map();
        // 섹션별 유효한 하위 항목 분류
        const h1Entries = new Map(); // 최상위 항목
        const childEntries = new Map(); // 상위 해시 -> 하위 해시 세트
        // 1. 먼저 모든 H1 항목을 수집하고 맵에 저장
        for (const entry of validatedToc) {
            if (entry.level === 1) {
                h1Entries.set(entry.hash, entry);
                childEntries.set(entry.hash, new Set());
            }
        }
        // 2. 각 항목을 처리하여 해시 기반으로 적절한 부모-자식 관계 검증
        for (const entry of validatedToc) {
            // H1은 이미 처리됨
            if (entry.level === 1) {
                uniqueMap.set(entry.hash, entry);
                continue;
            }
            // H2, H3 항목 처리
            if (entry.parentHash) {
                // 부모 해시가 있는 경우 부모가 실제로 존재하는지 확인
                const parentExists = uniqueMap.has(entry.parentHash) || h1Entries.has(entry.parentHash);
                if (parentExists) {
                    // 부모에 이 항목을 자식으로 추가
                    if (childEntries.has(entry.parentHash)) {
                        childEntries.get(entry.parentHash).add(entry.hash);
                    }
                    else {
                        childEntries.set(entry.parentHash, new Set([entry.hash]));
                    }
                    // 항목을 유효한 것으로 추가
                    uniqueMap.set(entry.hash, entry);
                }
                else {
                    // 부모가 존재하지 않는 고아 항목
                    reporter.warn(`Skipping orphaned TOC entry: ${entry.title} (parent hash ${entry.parentHash} not found)`);
                }
            }
            else if (entry.level === 2) {
                // H2 항목이지만 부모 해시가 없는 경우 (비정상)
                reporter.warn(`Skipping H2 entry without parent: ${entry.title}`);
            }
            else {
                // 부모 해시가 없지만 유효할 수 있는 항목
                uniqueMap.set(entry.hash, entry);
            }
        }
        // 내부 일관성 검사: 동일한 제목의 항목이 여러 섹션에 있는 경우
        const titleToSections = new Map();
        for (const entry of uniqueMap.values()) {
            if (entry.level === 2) {
                const title = entry.title.toLowerCase();
                if (!titleToSections.has(title)) {
                    titleToSections.set(title, new Set());
                }
                // 이 제목이 속한 부모 H1 추적
                if (entry.parentHash) {
                    titleToSections.get(title).add(entry.parentHash);
                }
            }
        }
        // 여러 섹션에 동일한 제목의 H2가 있는지 확인하고, 영향받는 H2의 contextTitle을 더 강화
        for (const [title, sections] of titleToSections.entries()) {
            if (sections.size > 1) {
                reporter.info(`Title "${title}" appears across ${sections.size} different sections. Enhancing context.`);
                // 해당 제목의 모든 항목에 대해 상위 항목의 제목을 컨텍스트에 추가
                for (const entry of uniqueMap.values()) {
                    if (entry.title.toLowerCase() === title && entry.parentHash) {
                        const parentEntry = h1Entries.get(entry.parentHash);
                        if (parentEntry) {
                            entry.contextTitle = `${entry.title} (${parentEntry.title})`;
                        }
                    }
                }
            }
        }
        // 최종 결과를 배열로 변환
        processedToc = Array.from(uniqueMap.values());
        const removedCount = validatedToc.length - processedToc.length;
        const processTime = Date.now() - startTime;
        if (removedCount > 0) {
            reporter.info(`Removed ${removedCount} duplicate/invalid TOC entries in ${processTime}ms.`);
        }
    }
    // Add context to duplicate title entries for better display
    if (enhanceDuplicates) {
        (0, exports.enrichDuplicateTitles)(processedToc, reporter);
    }
    // Limit size if necessary
    if (processedToc.length > maxSize) {
        reporter.warn(`Table of contents exceeds maximum size (${processedToc.length} > ${maxSize}). Truncating.`);
        processedToc = processedToc.slice(0, maxSize);
    }
    // Build the hierarchical structure if requested
    if (buildHierarchy) {
        // First level sort - by level, then by approximate order
        processedToc.sort((a, b) => {
            if ((a.level || 0) !== (b.level || 0)) {
                return (a.level || 0) - (b.level || 0);
            }
            return 0; // Keep original order for same level
        });
        // 확인을 위한 로깅
        const h1Count = processedToc.filter((item) => item.level === 1).length;
        const h2Count = processedToc.filter((item) => item.level === 2).length;
        const h3Count = processedToc.filter((item) => item.level === 3).length;
        reporter.info(`TOC structure: ${h1Count} h1, ${h2Count} h2, ${h3Count} h3 headings`);
    }
    return processedToc;
};
exports.optimizeTocArray = optimizeTocArray;
/**
 * TOC 구조의 유효성을 검사하고 문제가 있는 항목을 필터링
 */
const validateTocStructure = (tocEntries, reporter) => {
    if (!tocEntries || tocEntries.length === 0) {
        return [];
    }
    // 1. 섹션별 매핑 구성
    const h1Map = new Map();
    const h1Children = new Map();
    // 2. 모든 H1 항목 수집
    for (const entry of tocEntries) {
        if (entry.level === 1) {
            h1Map.set(entry.hash, entry);
            h1Children.set(entry.hash, new Set());
        }
    }
    // H1이 없으면 원본 반환
    if (h1Map.size === 0) {
        return tocEntries;
    }
    // 3. 각 항목을 올바른 부모에 연결
    for (const entry of tocEntries) {
        if (entry.level !== 1 && entry.parentHash) {
            // 부모가 H1인 경우
            if (h1Map.has(entry.parentHash)) {
                h1Children.get(entry.parentHash).add(entry.hash);
            }
        }
    }
    // 4. 각 H1 섹션 내에서 중복된 H2/H3 타이틀 발견 (같은 섹션에 동일 타이틀 중복)
    const suspiciousEntries = new Set();
    const sectionTitles = new Map();
    for (const [h1Hash, childHashes] of h1Children.entries()) {
        const titleCounts = new Map();
        sectionTitles.set(h1Hash, titleCounts);
        // 이 H1에 속한 모든 자식 항목 검사
        for (const entry of tocEntries) {
            if ((entry.level === 2 || entry.level === 3) &&
                entry.parentHash === h1Hash) {
                const title = entry.title.toLowerCase();
                const count = (titleCounts.get(title) || 0) + 1;
                titleCounts.set(title, count);
                // 이 섹션에 이 제목이 이미 있으면 중복으로 표시
                if (count > 1) {
                    suspiciousEntries.add(entry.hash);
                }
            }
        }
    }
    // 5. 다른 섹션에 속할 것으로 보이는 항목 식별 (예: 14. Squid 프록시 서버의 잘못된 자식)
    const titleToSections = new Map();
    for (const entry of tocEntries) {
        if (entry.level === 2) {
            const title = entry.title.toLowerCase();
            if (!titleToSections.has(title)) {
                titleToSections.set(title, new Set());
            }
            // 이 제목이 어떤 H1 섹션에 속하는지 기록
            if (entry.parentHash) {
                titleToSections.get(title).add(entry.parentHash);
            }
        }
    }
    // 여러 섹션에 동일한 제목의 항목이 있는지 검사
    for (const [title, sections] of titleToSections.entries()) {
        if (sections.size > 1) {
            reporter.info(`Title "${title}" appears across ${sections.size} different sections.`);
            // 일반적인 명령어 이름인지 확인 (cp, mv, rm 등)
            const isCommonCommand = /^(cp|mv|rm|find|grep|curl|wget|ls|chmod|chown|ps|sed|awk|cat|service|file|locate|stat)$/.test(title);
            if (isCommonCommand) {
                reporter.info(`"${title}" is a common command name that appears in multiple sections.`);
                // 각 섹션의 항목 분석
                for (const entry of tocEntries) {
                    if (entry.title.toLowerCase() === title && entry.level === 2) {
                        // 이 H2 항목이 이 섹션의 내용과 일치하는지 확인할 방법이 필요
                        // 예: 이 H2와 연결된 H3 항목이 정당한지 검사
                        let hasValidChildren = false;
                        for (const child of tocEntries) {
                            if (child.level === 3 && child.parentHash === entry.hash) {
                                hasValidChildren = true;
                                break;
                            }
                        }
                        // 자식이 없는 중복된 명령어는 다른 섹션에서 복사된 것일 가능성이 높음
                        if (!hasValidChildren && sections.size > 1) {
                            suspiciousEntries.add(entry.hash);
                        }
                    }
                }
            }
        }
    }
    // 6. 의심스러운 항목 수 보고
    if (suspiciousEntries.size > 0) {
        reporter.info(`Found ${suspiciousEntries.size} suspicious TOC entries that might be duplicates`);
    }
    // 7. 의심스러운 항목 제거된 결과 반환
    return tocEntries.filter((entry) => !suspiciousEntries.has(entry.hash));
};
exports.validateTocStructure = validateTocStructure;
/**
 * Enhances TOC entries with contextual titles to disambiguate duplicate titles
 */
const enrichDuplicateTitles = (tocEntries, reporter) => {
    if (!tocEntries || tocEntries.length === 0)
        return;
    // Find duplicate titles
    const titleCounts = new Map();
    const duplicateTitles = new Set();
    // Count occurrences of each title
    for (const entry of tocEntries) {
        const count = (titleCounts.get(entry.title) || 0) + 1;
        titleCounts.set(entry.title, count);
        if (count > 1) {
            duplicateTitles.add(entry.title);
        }
    }
    if (duplicateTitles.size === 0)
        return;
    reporter.info(`Found ${duplicateTitles.size} duplicate title(s) in TOC. Adding context...`);
    // Build parent map for fast lookups
    const entryMap = new Map();
    for (const entry of tocEntries) {
        entryMap.set(entry.hash, entry);
    }
    // Process each entry to ensure all duplicate titles have context
    for (const entry of tocEntries) {
        if (duplicateTitles.has(entry.title) && !entry.contextTitle) {
            let context = "";
            let parentEntry;
            // Find parent context
            if (entry.parentHash && entryMap.has(entry.parentHash)) {
                parentEntry = entryMap.get(entry.parentHash);
                context = parentEntry?.title || "";
            }
            // Add context to title
            if (context) {
                entry.contextTitle = `${entry.title} (${context})`;
            }
            else {
                entry.contextTitle = entry.title;
            }
        }
    }
};
exports.enrichDuplicateTitles = enrichDuplicateTitles;
